@@ -247,7 +247,7 @@
 #' \emph{Applied Mathematics and Computation}, \strong{436}. \cr
 #' DOI: \doi{10.1016/j.amc.2022.127493}
 #' 
-#' Dimitrova, D. S., Guillen, E. S. and Kaishev, V. K.  (2024).
+#' Dimitrova, D. S., Kaishev, V. K. and Saenz Guillen, E. (2024).
 #' \pkg{GeDS}: An \proglang{R} Package for Regression, Generalized Additive
 #' Models and Functional Gradient Boosting, based on Geometrically Designed
 #' (GeD) Splines. \emph{Manuscript submitted for publication.}
@@ -995,7 +995,7 @@ componentwise_fit <- function(bl_name, response, data, model_formula_template, f
           NGeDS(model_formula, data = data, weights = weights, beta = beta, phi = phi,
                 min.intknots = 0, max.intknots = max.intknots, q = q,
                 Xextr = NULL, Yextr = NULL, show.iters = FALSE, stoptype = "RD",
-                higher_order = FALSE, intknots = starting_intknots, only_pred = TRUE)
+                higher_order = FALSE, intknots_init = starting_intknots, only_pred = TRUE)
           } else {
             GGeDS(model_formula, data = data, family = family, weights = weights,
                   beta = beta, phi = phi, min.intknots = 0, max.intknots = max.intknots,
