@@ -33,6 +33,7 @@ validate_iterations <- function(iterations, default, name) {
 #########################
 #' @import mboost
 #' @import stats
+#' @import mi
 
 # Input should be "family`at`name", with family being a boost_family_glm object
 get_mboost_family <- function(family) {
@@ -53,6 +54,7 @@ get_mboost_family <- function(family) {
          "Negative Binomial Likelihood -- cloglog link" = stats::binomial(link = "cloglog"),
          # Poisson
          "Poisson Likelihood" = stats::poisson(link = "log"),
+         "Negative Multinomial Likelihood" = mi::multinomial(link = "logit"),
          stop("Invalid distribution type")
   )
 }
